<?php

namespace Drupal\change_labels\EventSubscriber;

use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\core_event_dispatcher\Event\Form\FormAlterEvent;
use Drupal\core_event_dispatcher\Event\Form\FormBaseAlterEvent;
use Drupal\core_event_dispatcher\FormHookEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Change labels event subscriber.
 */
class ChangeSubmitLabelSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      'hook_event_dispatcher.form_base_entity_form_display_form.alter' => ['addSettingsForm'],
      FormHookEvents::FORM_ALTER => ['alterForm'],
    ];
  }

  /**
   * Event handler for the third party settings of a widget.
   *
   * @param \Drupal\core_event_dispatcher\Event\Form\FormBaseAlterEvent $event
   *   Triggering event.
   */
  public function addSettingsForm(FormBaseAlterEvent $event) {
    $entityFormDisplay = $event->getFormState()->getFormObject()->getEntity();
    $form = &$event->getForm();
    $form['change_labels'] = [
      '#type' => 'details',
      '#title' => $this->t('Change labels'),
    ];
    $form['change_labels']['submit_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Button label for "Submit"'),
      '#default_value' => $entityFormDisplay->getThirdPartySetting('change_labels', 'submit_label'),
    ];
    $form['change_labels']['submit_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Submit message replacement'),
      '#description' => $this->t('EXPERIMENTAL: setting a value here will supress all status messages like "NODE_TYPE TITLE has been created" from node module.'),
      '#default_value' => $entityFormDisplay->getThirdPartySetting('change_labels', 'submit_message'),
    ];
    $form['#entity_builders'][] = [$this, 'saveThirdPartySettings'];
  }

  /**
   * Entity Builder Callback.
   */
  public function saveThirdPartySettings($entityType, $entityFormDisplay, array &$form, FormStateInterface $formState) {
    foreach (['submit_label', 'submit_message'] as $field) {
      if ($formState->getValue($field)) {
        $entityFormDisplay->setThirdPartySetting('change_labels', $field, $formState->getValue($field));
      }
      else {
        $entityFormDisplay->unsetThirdPartySetting('change_labels', $field);
      }
    }
  }

  /**
   * Event handler for altering the submit label of a form.
   *
   * @param \Drupal\core_event_dispatcher\Event\Form\FormAlterEvent $event
   *   Triggering event.
   */
  public function alterForm(FormAlterEvent $event) {
    $formObject = $event->getFormState()->getFormObject();
    $form = &$event->getForm();
    if ($formObject instanceof ContentEntityFormInterface &&
      ($display = $formObject->getFormDisplay($event->getFormState())) &&
      isset($form['actions']['submit']['#value'])
    ) {
      if ($submitReplacement = $display->getThirdPartySetting('change_labels', 'submit_label')) {
        $form['actions']['submit']['#value'] = $submitReplacement;
      }
      if ($display->getThirdPartySetting('change_labels', 'submit_message')) {
        $form['actions']['submit']['#submit'][] = static::class . '::replaceMessage';
      }
    }
  }

  /**
   * Change message when a node is submitted.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   Form state.
   */
  public static function replaceMessage(array $form, FormStateInterface $formState): void {
    $formObject = $formState->getFormObject();
    if (
      !$formObject instanceof ContentEntityFormInterface ||
      !($display = $formObject->getFormDisplay($formState))
    ) {
      return;
    }
    if ($submitMessageReplacement = $display->getThirdPartySetting('change_labels', 'submit_message')) {
      // Remove all status messages.
      $messenger = \Drupal::messenger();
      $messenger->deleteByType($messenger::TYPE_STATUS);
      $messenger->addStatus($submitMessageReplacement);
    }
  }

}
